--
-- $RCSfile: ALU_32.vhd,v $
-- $Revision: 1.4 $
-- $Author: petera $
-- $Date: 90/06/22 12:52:31 $
--

use work.dp32_types.all, work.ALU_32_types.all;

entity ALU_32 is
  generic (Tpd : Time := unit_delay);
  port (operand1 : in bit_32;
      	operand2 : in bit_32;
	result : out bus_bit_32 bus;
	cond_code : out CC_bits;
	command : in ALU_command);
end ALU_32;

architecture behaviour of ALU_32 is

  alias cc_V : bit is cond_code(2);
  alias cc_N : bit is cond_code(1);
  alias cc_Z : bit is cond_code(0);
  
begin

  ALU_function: process (operand1, operand2, command)

    variable a, b : integer;
    variable temp_result : bit_32;

  begin
    case command is
      when add | subtract | multiply | divide =>
      	a := bits_to_int(operand1);
      	b := bits_to_int(operand2);
      when incr1 =>
      	a := bits_to_int(operand1);
      	b := 1;
      when others =>
      	null;
    end case;
    case command is
      when disable =>
      	null;
      when pass1 =>
      	temp_result := operand1;
      when log_and =>
       	temp_result := operand1 and operand2;
      when log_or =>
       	temp_result := operand1 or operand2;
      when log_xor =>
       	temp_result := operand1 xor operand2;
      when log_mask =>
       	temp_result := operand1 and not operand2;
      when add | incr1 =>
      	if b > 0 and a > integer'high-b then  -- positive overflow
	  int_to_bits(((integer'low+a)+b)-integer'high-1, temp_result);
	  cc_V <= '1' after Tpd;
	elsif b < 0 and a < integer'low-b then  -- negative overflow
	   int_to_bits(((integer'high+a)+b)-integer'low+1, temp_result);
	  cc_V <= '1' after Tpd;
	else
	  int_to_bits(a + b, temp_result);
	  cc_V <= '0' after Tpd;
	end if;
      when subtract =>
      	if b < 0 and a > integer'high+b then  -- positive overflow
	  int_to_bits(((integer'low+a)-b)-integer'high-1, temp_result);
	  cc_V <= '1' after Tpd;
	elsif b > 0 and a < integer'low+b then  -- negative overflow
	  int_to_bits(((integer'high+a)-b)-integer'low+1, temp_result);
	  cc_V <= '1' after Tpd;
	else
	  int_to_bits(a - b, temp_result);
	  cc_V <= '0' after Tpd;
	end if;
      when multiply =>
      	if ((a>0 and b>0) or (a<0 and b<0))  -- result positive
	    and (abs a > integer'high / abs b) then  -- positive overflow
	  int_to_bits(integer'high, temp_result);
	  cc_V <= '1' after Tpd;
	elsif ((a>0 and b<0) or (a<0 and b>0))  -- result negative
      	    and ((- abs a) < integer'low / abs b) then  -- negative overflow
	  int_to_bits(integer'low, temp_result);
	  cc_V <= '1' after Tpd;
	else
	  int_to_bits(a * b, temp_result);
	  cc_V <= '0' after Tpd;
	end if;
      when divide =>
      	if b=0 then
	  if a>=0 then 			  -- positive overflow
	    int_to_bits(integer'high, temp_result);
	  else
	    int_to_bits(integer'low, temp_result);
	  end if;
	  cc_V <= '1' after Tpd;
	else
	  int_to_bits(a / b, temp_result);
	  cc_V <= '0' after Tpd;
	end if;
    end case;
    if command /= disable then
      result <= temp_result after Tpd;
    else
      result <= null after Tpd;
    end if;
    cc_Z <= bool_to_bit(temp_result = X"00000000") after Tpd;
    cc_N <= bool_to_bit(temp_result(31) = '1') after Tpd;
  end process ALU_function;
  
end behaviour;
